; NeoGeo CD CDDA track player (2) with sound visualisation for JAS
; CC 2011 furrtek - furrtek.org
; http://neogeodev.wikkii.com/

    INCLUDE "regdefs.asm"
    INCLUDE "cdheader.asm"

    VBLC                  equ           RAMSTART           ; Frame counter
    INDX                  equ           RAMSTART+2         ; Current buffer index
    PALOFFS               equ           RAMSTART+4         ; Palette rotator offset
    PALINDX               equ           RAMSTART+6         ; Current palette index

    NEWLEFTLEVELS         equ           RAMSTART+8         ; Update buffer
    OLDLEFTLEVELS         equ           RAMSTART+8+256     ; Display buffer

    FLAGS                 equ           RAMSTART+8+512

VBlank:
    btst    #7,BIOS_SYSSTAT
    bne     DisplayedLogo
    move.l  #BIOSF_BOOTSCR,a0   ; Classic stuff
    jmp     (a0)
DisplayedLogo:
    movem.l d0-d7/a0-a6,-(a7)
    move.w  #4,REG_IRQACK       ; Better ack at the end ?
    move.b  d0,REG_DIPSW        ; Kick watchdog
    btst.b  #0,FLAGS
    beq     noanim

    lea     NEWLEFTLEVELS,a0    ; Flip level buffers
    lea     OLDLEFTLEVELS,a1
    move.l  #64-1,d0
fliplevels:
    move.w  (a0)+,(a1)+
    dbra    d0,fliplevels

    move.w  #FIXMAP+(23+2+((8+1)*32)),VRAM_ADDR
    lea     VBLC,a0
    jsr     WriteHex            ; Display frame counter

    lea     OLDLEFTLEVELS,a0
    move.l  #1,d2
    move.w  #1,VRAM_MOD
    move.l  #64-1,d7
    clr.l   d4
barslp:
    clr.l   d1                  ; Convert level value to binary and add it up in D4
    move.w  (a0)+,d1
    jsr     conv
    add.l   d1,d4

    move.w  d1,d0               ; Save current level

    move.l  d2,d3               ; Use it for shrinking sprite vertically
    addi.w  #SCB2,d3
    move.w  d3,VRAM_ADDR        ; Compute address for sprite #2 in SCB2
    lsr     #7,d1
    ori.w   #$0F00,d1           ; No horizontal shrinking
    move.w  d1,VRAM_RW

    move.w  d0,d1               ; Get current level back
    lsr     #8,d1               ; Adjust for Y sprite position centering
    lsr     #2,d1
    neg.w   d1
    addi.w  #(96/2)+80,d1       ; D1=((96/2)+80)-D1
    move.w  d1,d0

    move.l  d2,d3
    addi.w  #SCB3,d3
    move.w  d3,VRAM_ADDR        ; Compute address for sprite #2 in SCB3
    jsr     SetSprY

    addq.w  #1,d2               ; Next sprite
    dbra    d7,barslp           ; Next level

    lsr.l   #6,d4               ; Make average out of all levels
    lsr.l   #3,d4
    move.w  PALOFFS,d0
    lsr.w   #8,d0               ; 8.8 fixed point to integer
    andi.w  #$3E,d0             ; Loop every 32, 16bit values
    move.b  d0,PALINDX
    add.w   d4,PALOFFS          ; Add to make the colors "scroll"

    clr.w   INDX                ; Start each frame with buffer index 0

    move.l  #BIOSF_VBL,a0       ; BIOS call for a lot of important things
    jsr     (a0)

    cmp.w   #$60,VBLC           ; Skip the first $60 frames for line interruptions (why ?)
    bne     noanim
    move    #$180*4,$3C000A	; Interrupt every ~4 raster lines
    move    #$0000,REG_HBLANKPOS
    move    #$00D0,REG_HBLANKCNT
noanim:

    addq.w  #1,VBLC
    movem.l (a7)+,d0-d7/a0-a6
    rte

IRQ2:
    move.w  #2,REG_IRQACK
    movem.l d0-d7/a0-a6,-(a7)

    clr.l   d0                  ; Get and set current line background color
    move.b  PALINDX,d0
    lea     PurpleSine,a0
    move.w  0(a0,d0.w),d1
    move.w  d1,PALETTES+(16*2*256)-2
    addq.b  #2,PALINDX          ; Palette entries are 16bits
    andi.b  #$3F,PALINDX        ; Loop every 32, 16bit values

    clr.l   d1
    move.w  REG_CDDALEFTL,d1
    lea     NEWLEFTLEVELS,a0    ; Get the left channel's level
    move.w  INDX,d0
    addq.w  #2,INDX             ; Level entries are 16bits
    move.w  d1,0(a0,d0.w)

    movem.l (a7)+,d0-d7/a0-a6
    rte

IRQ3:
    move.w  #1, REG_IRQACK
    rte

Start:
    lea     $10F300, sp
    lea     $10F300, a7         ; Depends on assembler
    move.b  d0,REG_DIPSW        ; Kick watchdog
    clr.b   FLAGS               ; Disable animation for now
    move.w  #7,REG_IRQACK
    move.w  #$2000,sr           ; Enable interrupts

    move.b  #$01,REG_SOUND      ; Init sound driver (shouldn't be useful)

    move.w  #0,VBLC             ; Clear frame counter

    move.l  #(40*32)-1,d0       ; Clears fix layer with tile $FF, palette 0
    move.w  #FIXMAP,VRAM_ADDR
    move.w  #$0001,VRAM_MOD
clfix:
    move.w  #$00FF,VRAM_RW
    nop
    dbra    d0,clfix

    lea     Palette,a0          ; Loads one palette
    lea     PALETTES,a1
    move.l  #16-1,d0
ldpalt:
    move.w  (a0)+,(a1)+
    dbra    d0,ldpalt

    lea     PALETTES+(16*2*256)-2,a0
    move.w  #BLACK,(a0)         ; Sets default backdrop color (will be overwritten later)

    move.w  #SCB3,VRAM_ADDR     ; Clears sprite maps
    move.w  #1,VRAM_MOD
    move.l  #380-1,d7
clspr:
    move.w  #0,VRAM_RW
    dbra    d7,clspr

    move.l  #1,d0               ; Sets up 64 sprites as vertical bars
    move.l  #64-1,d7
setsprites:
    move.l  d0,d1
    lsl.w   #6,d1               ; Compute address for sprite d0 in SCB1
    move.w  d1,VRAM_ADDR        ; Set 4 tiles in the sprite's map:
    move.w  #1,VRAM_RW          ; Tile 1
    move.w  #0,VRAM_RW          ; Palette 0, no special attributes
    move.w  #1,VRAM_RW          ; ...
    move.w  #0,VRAM_RW
    move.w  #1,VRAM_RW
    move.w  #0,VRAM_RW
    move.w  #1,VRAM_RW
    move.w  #0,VRAM_RW
    move.l  d0,d1
    add.w   #SCB2,d1            ; Compute address for sprite d0 in SCB2
    move.w  d1,VRAM_ADDR
    move.w  #$0F40,VRAM_RW      ; Set sprite's zoom to test vertical shrinking
    move.l  d0,d1
    add.w   #SCB3,d1            ; Compute address for sprite d0 in SCB3
    move.w  d1,VRAM_ADDR
    move.w  #((496-80)<<7)+4,VRAM_RW     ; Set sprite's initial Y position and height
    move.l  d0,d1
    add.w   #SCB4,d1            ; Compute address for sprite d0 in SCB4
    move.w  d1,VRAM_ADDR
    move.l  d0,d1
    lsl.w   #2,d1               ; X=Sprite #*5 (sprites only contain a 4 pixel wide strip)
    add.w   d0,d1
    lsl.w   #7,d1
    move.w  d1,VRAM_RW
    addq.w  #1,d0               ; Next sprite
    dbra    d7,setsprites

    move.b  d0,REG_DIPSW        ; Watchdog (shouldn't be useful, interrupts are running)
    move.l  #Sprite,BIOS_UPSRC  ; Upload a sprite tile in tile #1 (leave tile #0 blank)
    move.l  #128,BIOS_UPDEST    ; Sprites tiles are 128 bytes
    move.b  #2,BIOS_UPZONE      ; SPR zone
    move.l  #128,BIOS_UPSIZE
    move.b  #0,BIOS_UPBANK      ; First bank
    move.l  #BIOSF_UPLOAD,a0
    jsr     (a0)

    clr.b   PALOFFS             ; Init palette animation
    clr.b   PALINDX

    move.b  #1,FLAGS            ; Allows animation

    clr.w   d0
    move.w  #$0502,d0           ; Starts playing CDDA track 2 with no looping
    move.l  #BIOSF_CDDACMD,a0
    jsr     (a0)

Loop:
   bra     Loop

Palette:                        ; Debug palette
    dc.w $8000,$7FFF,$0F00,$00F0
    dc.w $8000,$7FFF,$0F00,$00F0
    dc.w $8000,$7FFF,$0F00,$00F0
    dc.w $8000,$7FFF,$0F00,$0FA0

;Displays 16bit value pointed by A0. Please set VRAM fix address+3 before, thanks.
WriteHex:
    move.l  d0,-(a7)
    move.w  #$FFFF-$20+1, VRAM_MOD ; -32
    move.w  (a0),d0
    andi.w  #$000F,d0
    jsr     hexshift
    addi.w  #$30,d0             ; Numbers start in fix tileset
    move.w  d0, VRAM_RW         ; LSB low nibble

    move.w  (a0),d0
    andi.w  #$00F0,d0
    ror.w   #4,d0
    jsr     hexshift
    addi.w  #$30,d0
    move.w  d0, VRAM_RW         ; LSB high nibble

    move.w  (a0),d0
    andi.w  #$0F00,d0
    ror.w   #8,d0
    jsr     hexshift
    addi.w  #$30,d0
    move.w  d0, VRAM_RW         ; MSB low nibble

    move.w  (a0),d0
    rol.w   #4,d0
    andi.w  #$000F,d0
    jsr     hexshift
    addi.w  #$30,d0
    move.w  d0, VRAM_RW
    move.l  (a7)+,d0            ; MSB high nibble
    rts

hexshift:
    cmp.b   #$A,d0              ; Switch to letters if > 9
    blt.b   r
    addq.w  #7,d0
r:
    rts

; Converts reversed 16bit 2's complement value to binary
conv:
    move.l  d7,-(a7)
    moveq   #16-1,d7            ; 16bits to reverse
    moveq   #0,d0
rev:
    lsr	    #1,d1
    bcc	    skip
    bset    d7,d0               ; What a nice instruction
skip:
    dbf	    d7,rev
    move.l  d0,d1
    tst     d1
    bpl     notneg              ; abs()
    not     d1
notneg:	
    move.l  (a7)+,d7
    rts
  
; Sets sprite Y position (VRAM address needs to be set before)
SetSprY:
    neg.w   d0
    addi.w  #496,d0
    lsl.w   #7,d0
    add     #4,d0               ; ORs sprite height
    move.w  d0,VRAM_RW
    rts
    
Sprite:                         ; Sprite tile filled with a 4 pixel wide bar on the right, color $F
    dc.b    $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
    dc.b    $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
    dc.b    $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
    dc.b    $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
    dc.b    $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
    dc.b    $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
    dc.b    $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
    dc.b    $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
    dc.b    $00,$00,$00,$00,$00,$00,$00,$00
    dc.b    $00,$00,$00,$00,$00,$00,$00,$00
    dc.b    $00,$00,$00,$00,$00,$00,$00,$00
    dc.b    $00,$00,$00,$00,$00,$00,$00,$00
    dc.b    $00,$00,$00,$00,$00,$00,$00,$00
    dc.b    $00,$00,$00,$00,$00,$00,$00,$00
    dc.b    $00,$00,$00,$00,$00,$00,$00,$00
    dc.b    $00,$00,$00,$00,$00,$00,$00,$00
    
PurpleSine:                     ; Palette values
    dc.w    $0000,$0001,$0102,$0203,$0305,$0406,$0507,$0608
    dc.w    $0608,$0709,$0709,$080A,$080A,$080A,$090B,$090B
    dc.w    $090B,$090B,$080A,$080A,$080A,$0709,$0709,$0608
    dc.w    $0608,$0507,$0406,$0305,$0203,$0102,$0001,$0000
